# -*- coding: utf-8 -*-

ACCESS_KEY = "<access_key>"
SECRET_KEY = "<secret_key>"

RS_HOST = "rs.qbox.me"
RSF_HOST = "rsf.qbox.me"
UP_HOST = "up.qiniu.com"

from . import __version__
USER_AGENT = "qiniu python-sdk v%s" % __version__
